@@ -1,14 +1,10 @@
 use rusoto_sqs::Message as SqsMessage;
 use tokio::sync::mpsc::{channel, Receiver, Sender};
-use log::*;
 
-use crate::event_handler::{EventHandler, OutputEvent, Completion};
-use crate::event_retriever::PayloadRetriever;
+use crate::event_handler::EventHandler;
+use crate::event_retriever::EventRetriever;
 use crate::sqs_completion_handler::CompletionHandler;
 use crate::sqs_consumer::Consumer;
-use std::fmt::Debug;
-
-use std::marker::PhantomData;
 
 #[derive(Copy, Clone, Debug)]
 pub enum ProcessorState {
@@ -24,15 +20,14 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     consumer: C,
     completion_handler: CH,
     event_retriever: ER,
     event_handler: EH,
     state: ProcessorState,
-    self_actor: Option<EventProcessorActor>,
 }
 
 impl<C, EH, Input, Output, ER, CH> EventProcessor<C, EH, Input, Output, ER, CH>
@@ -41,9 +36,8 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-    
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     pub fn new(
         consumer: C,
@@ -57,8 +51,6 @@ where
             event_handler,
             event_retriever,
             state: ProcessorState::Waiting,
-            self_actor: None,
-
         }
     }
 }
@@ -69,40 +61,42 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-    
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     pub async fn process_event(&mut self, event: SqsMessage) {
         // TODO: Handle errors
-        info!("Retrieved event");
         let retrieved_event = match self.event_retriever.retrieve_event(&event).await {
             Ok(retrieved_event) => retrieved_event,
-            Err(e) => {
-                warn!("Failed to retrieve event with: {:?}", e);
+            Err(_e) => {
                 return
                 // TODO: Retry
                 // TODO: We could reset the message visibility to 0 so it gets picked up again?
             }
         };
 
-        info!("Handling event");
-        let output_event = self.event_handler.handle_event(retrieved_event).await;
+        let completed = match self.event_handler.handle_event(retrieved_event).await {
+            Ok(completed) => completed,
+            Err(_e) => {
+                return
+                // TODO: Retry
+                // TODO: We could reset the message visibility to 0 so it gets picked up again?
+            }
+        };
 
-        self.completion_handler.mark_complete(event, output_event).await;
+        self.completion_handler.mark_complete(event, completed).await;
 
         if let ProcessorState::Started = self.state {
             self.consumer
-                .get_next_event(self.self_actor.clone().unwrap()).await;
+                .get_next_event(EventProcessorActor::new(self.clone())).await;
         }
     }
 
     pub async fn start_processing(&mut self) {
         self.state = ProcessorState::Started;
 
-        info!("Getting next event from consumer");
         self.consumer
-            .get_next_event(self.self_actor.clone().unwrap()).await;
+            .get_next_event(EventProcessorActor::new(self.clone())).await;
     }
 
     pub fn stop_processing(&mut self) {
@@ -123,10 +117,8 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-
-    
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     pub async fn route_message(&mut self, msg: EventProcessorMessage) {
         match msg {
@@ -143,21 +135,16 @@ pub struct EventProcessorActor {
 }
 
 impl EventProcessorActor {
-    pub fn new<C, EH, Input, Output, ER, CH>(mut actor_impl: EventProcessor<C, EH, Input, Output, ER, CH>) -> Self
+    pub fn new<C, EH, Input, Output, ER, CH>(actor_impl: EventProcessor<C, EH, Input, Output, ER, CH>) -> Self
     where
         C: Consumer + Clone + Send + Sync + 'static,
         EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
         Input: Send + Clone + 'static,
         Output: Send + Sync + Clone + 'static,
-        ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-        CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-
-        
+        ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+        CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
     {
-        let (sender, receiver) = channel(1);
-
-        let self_actor = Self {sender};
-        actor_impl.self_actor = Some(self_actor.clone());
+        let (sender, receiver) = channel(0);
 
         tokio::task::spawn(
             route_wrapper(
@@ -168,7 +155,7 @@ impl EventProcessorActor {
             )
         );
 
-        self_actor
+        Self { sender }
     }
 
     pub async fn process_event(&self, event: SqsMessage) {
@@ -199,10 +186,8 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-
-    
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     receiver: Receiver<EventProcessorMessage>,
     actor_impl: EventProcessor<C, EH, Input, Output, ER, CH>,
@@ -215,10 +200,8 @@ where
     EH: EventHandler<InputEvent = Input, OutputEvent = Output> + Send + Sync + Clone + 'static,
     Input: Send + Clone + 'static,
     Output: Send + Sync + Clone + 'static,
-    ER: PayloadRetriever<Input> + Send + Sync + Clone + 'static,
-    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=OutputEvent<Output, <EH as EventHandler>::Error>> + Send + Sync + Clone + 'static,
-    
-    
+    ER: EventRetriever<Input> + Send + Sync + Clone + 'static,
+    CH: CompletionHandler<Message=SqsMessage, CompletedEvent=Output> + Send + Sync + Clone + 'static,
 {
     while let Some(msg) = router.receiver.recv().await {
         router.actor_impl.route_message(msg).await;
